//$Id: winampFunc.cpp,v 1.17 2002/08/16 06:45:12 Yoshi Exp $
/*============================================================================

  Winamp2 control class
        copyright (C) 2000-2002 HOSHINO Yoshifumi,(Yoshi)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


% $Log: winampFunc.cpp,v $
% Revision 1.17  2002/08/16 06:45:12  Yoshi
% add GetPlayerStatus
%
% Revision 1.16  2002/03/18 09:46:59  yoshi
% (none)
%
% Revision 1.15  2002/03/10 10:17:30  yoshi
% (none)
%
% Revision 1.14  2002/01/25 01:25:34  yoshi
% delete GetTitle()
% add GetWindowTitle()
%
% Revision 1.8  2001/06/12 10:48:54  Yoshi
% add ExecuteEx()
%
% Revision 1.7  2001/05/18 09:36:15  Yoshi
% fixed some bags in init.
%
% Revision 1.6  2001/05/15 11:12:50  Yoshi
% added GetPlaybackTime.
% added GetPlaybackStatus.
% changed renamed some functions.
%
% Revision 1.5  2001/04/29 09:18:07  Yoshi
% merge "testver".
%
% Revision 1.4.2.2  2001/04/23 12:54:34  Yoshi
% changed own coding rules.
%
% Revision 1.4.2.1  2001/04/27 11:19:01  Yoshi
% changed a little 
%

2001-04-12
  added DisplayMainMenu()

2001-03-31
  added ampcmd.h for adding winamp commands easily.

2001-03-19
  added compile option. If you need no winamp3 functions, you can delete or commentout
  _WINAMP3_ALPHA is defined.

2001-03-11
  changed FindWinamp(), It searches winamp3 window first.

2001-02-22
  add #define _WINAMP3_ALPHA only.

2001-02-16(2)
  added definition _WINAMP3_ALPHA for selectable to use wa3 or not.
  deleted member lpszWinampPath. Function gets it whenever ExecuteWinamp() is called.

2001-02-16
  changed functions for Winamp3 algha4.(It's changed that window class name of wa3 alpha)
  changed str(r)chr to _tcs(r)chr for multibyte/doublebyte env.

2001-02-06
  added functions for Winamp3 alpha. We can use this to get window handle and
  window title about it. These will be enabled you set member bSenceWinamp3=TRUE.
  (It will be default in the future)

2000-10-06iuncompatible with previous versionj
  changed class constructer to be able to change default string.

2000/09/21
  gathered functions for winamp.

============================================================================*/

#define WIN32_LEAN_AND_MEAN
#define STRICT
	
#include <tchar.h>
#include <windows.h>
#include "winampFunc.h"


/*=========================================================
class init & destroy
=========================================================*/
CWinampFunction::CWinampFunction(LPSTR DefaultStr, LPSTR DefaultPath) : CPlayerDataContainer(WC_WINAMP2, DefaultStr, DefaultPath)
{
  m_bUseServer = FALSE;
}




/*=========================================================
GetPlayerStatus()
	This function get player statuses repeat/shuffle.

return
	unsigned int
  If repeat is on, GPLS_REPEAT flag is.
  If shuffle is on, GPLS_SHUFFLE flag is.
=========================================================*/
UINT CWinampFunction::GetPlayerStatus()
{
  HWND hwnd = FindWindow();

  if(!hwnd)
    return 0;

  UINT result=0;

  if(SendMessage(hwnd, WM_WA_IPC, 0, IPC_GET_REPEAT))
    result |= GPLS_REPEAT;

  if(SendMessage(hwnd, WM_WA_IPC, 0, IPC_GET_SHUFFLE))
    result |= GPLS_SHUFFLE;

  return result;
}




/*=========================================================
GetPlaybackStatus()
	This function gets playback status playing, pausing and so on.

return
	unsigned int
	GPS_PLAYING	(1)	Winamp is playing.
	GPS_PAUSED	(3)	Winamp is paused.
	GPS_STOPPED	(0)	Winamp is stopped.
	GPS_MISSING	(9)	Winamp is not running.
=========================================================*/
UINT CWinampFunction::GetPlaybackStatus()
{
  HWND hwnd = FindWindow();

  if(!hwnd)
    return GPS_MISSING;

  m_uiStatus = SendMessage(hwnd, WM_WA_IPC, 0, 104);

  return CPlayerDataContainer::GetPlaybackStatus();
}




/*=========================================================
GetWindowTitle(LPSTR, BOOL)
	This function gets winamp title as song info.

function
	LPSTR pointer to a buffer that receives title. 

return
	void
=========================================================*/

void CWinampFunction::GetWindowTitle(LPSTR TitleBuffer, size_t bufsize)
{
  char buffer[MAX_TITLE_SIZE];
  HWND hWnd;

  //find window
  if(NULL == ( hWnd = FindWindow()))
    strcpy(buffer, m_lpszDefaultTitle);
  else
    GetWindowText(hWnd , buffer, MAX_TITLE_SIZE);

  //copy to buffer
  strcpy(m_lpszTitle, buffer);
  if(TitleBuffer)
    strncpy(TitleBuffer, buffer, bufsize-1);
}




/*=========================================================
GetPlaybackTime(DWORD)
	This function gets some kinds of winamp time.

function
	DWORD some modes.

return long
	mode GPT_ELAPSED (0)
	returns elapsed time in seconds.
	mode GPT_REMAIN (1)
	returns remains time in seconds.
	mode GPT_TOTAL (10)
	returns current track length in seconds.
	mode GPT_ELAPSED_MILLISEC (20)
	returns elapsed time in milliseconds.
=========================================================*/
long CWinampFunction::GetPlaybackTime(DWORD mode)
{
  HWND hwinamp=FindWindow();

  if(!hwinamp)
    return 0;

  m_uiPlaybackTime = SendMessage(hwinamp, WM_WA_IPC, 0, IPC_GETOUTPUTTIME);
  m_uiPlaybackLength = SendMessage(hwinamp, WM_WA_IPC, 1, IPC_GETOUTPUTTIME) * 1000;

  return CPlayerDataContainer::GetPlaybackTime(mode);
}




/*=========================================================
SetFormat(char*, BOOL, BOOL)
	This function formats winamp title you got.

function
	char* Pointer to a buffer that set the title and will
            receive the formated title.
	BOOL  If It's TRUE, prefix will be deleted.
	BOOL  If It's TRUE, suffix will be deleted.

return void

=========================================================*/
void CWinampFunction::SetFormat(char *TitleBuffer, BOOL Prefix, BOOL Suffix)
{
  if(!strnicmp(TitleBuffer, "winamp", 6) || !strcmp(TitleBuffer , m_lpszDefaultTitle))
    return;

  TCHAR *tmpPtr;

  tmpPtr = _tcschr((_TCHAR *)TitleBuffer, '.');
  if(Prefix && tmpPtr)
    memmove(TitleBuffer, tmpPtr+2, strlen(tmpPtr));

  tmpPtr = _tcsrchr((_TCHAR *)TitleBuffer, '-');
  if(Suffix && tmpPtr){
    if(!strncmp(tmpPtr, DEFAULT_WINAMP_SUFFIX, sizeof(DEFAULT_WINAMP_SUFFIX)-1 ))
      *(tmpPtr -1) = NULL;
  }
}



/*=========================================================
PostCommand(DWORD)
	This function posts message as command to winamp.

function
	DWORD command for winampisee NSDN http://www.winamp.com/nsdnj

return 
	BOOL If success, returns TRUE.

===========================================================*/
BOOL CWinampFunction::PostCommand(DWORD command)
{
  m_dwLastCommand = command;

  HWND hWinamp=FindWindow();

  if(!hWinamp)
    return FALSE;

  return PostMessage(hWinamp, WM_COMMAND, (WPARAM)command, NULL);
}


/*=========================================================
GetWinampPath(LPSTR) **private**
	This function gets path to winamp.exe from registory.

function
	LPSTR Pointer to a buffer that you'll receive the path.

return 
	BOOL If success, returns TRUE.

=========================================================*/
BOOL CWinampFunction::GetPath(LPSTR AmpPath)
{
  HKEY hkey;
  DWORD dwType = REG_SZ,bufsize=MAX_PATH+1;
  char buffer[MAX_PATH+1];

  if(ERROR_SUCCESS != RegOpenKeyEx(HKEY_CLASSES_ROOT, "Winamp.File\\shell\\open\\command", 0, KEY_READ, &hkey)){
    strcpy(AmpPath, "");
    return FALSE;
  }

  if(ERROR_SUCCESS != RegQueryValueEx(hkey, NULL, NULL, &dwType, (LPBYTE)buffer, &bufsize)){
    strcpy(AmpPath, "");
    RegCloseKey(hkey);
    return FALSE;
  }

  RegCloseKey(hkey);

  if(buffer[0] == '\"'){
    *(char *)_tcschr((_TCHAR *)(buffer+1), '\"') = NULL;
    strncpy(AmpPath, buffer+1, MAX_PATH);
  } else {
    *(char *)_tcschr((_TCHAR *)(buffer+1), ' ') = NULL;
    strncpy(AmpPath, buffer, MAX_PATH);
  }

  return TRUE;
}


/*=========================================================
Execute(HWND)

=========================================================*/
/*inlined*/


/*=========================================================
DisplayMainMenu

=========================================================*/
void CWinampFunction::DisplayMainMenu()
{
  HWND hwnd = FindWindow();

  if(hwnd){
    SetForegroundWindow(hwnd);
    PostMessage(hwnd, WM_RBUTTONUP, 0, 0);
  }
}